
#line 3 "lex.yy.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart(yyin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int yyleng;

extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (yytext_ptr)  )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart (FILE *input_file  );
void yy_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE yy_create_buffer (FILE *file,int size  );
void yy_delete_buffer (YY_BUFFER_STATE b  );
void yy_flush_buffer (YY_BUFFER_STATE b  );
void yypush_buffer_state (YY_BUFFER_STATE new_buffer  );
void yypop_buffer_state (void );

static void yyensure_buffer_stack (void );
static void yy_load_buffer_state (void );
static void yy_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER yy_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE yy_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE yy_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE yy_scan_bytes (yyconst char *bytes,int len  );

void *yyalloc (yy_size_t  );
void *yyrealloc (void *,yy_size_t  );
void yyfree (void *  );

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer(yyin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;

typedef int yy_state_type;

extern int yylineno;

int yylineno = 1;

extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	(yytext_ptr) = yy_bp; \
	yyleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 85
#define YY_END_OF_BUFFER 86
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[249] =
    {   0,
       12,   12,    0,    0,   19,   19,   86,   16,   15,   14,
       13,   16,   12,   12,   11,   12,   12,   12,   12,   12,
       12,   12,   12,   84,   83,   41,   79,   73,   84,   37,
       38,   77,   76,   67,   75,   72,   78,   31,   31,   68,
       64,   39,   69,   40,   82,   36,   36,   70,   71,   80,
       36,   65,   81,   66,   74,   25,   19,   19,   25,   25,
       10,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   63,   51,   66,   42,   52,    0,    0,   49,   57,
       47,   58,   48,   59,    0,   34,   26,   27,   50,   35,
       30,    0,   31,    0,   31,   71,   65,   70,   56,   60,

       62,   61,   55,   36,    0,   53,   36,   54,   43,   19,
        0,    0,   17,   18,   10,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   32,   44,    0,   34,   34,    0,
       35,   30,    0,   33,   29,   46,   45,   36,    0,    0,
       22,   12,   12,   12,   12,   12,   12,   12,   12,   12,
        0,   34,    0,   34,    0,   35,   33,   29,   36,    0,
        0,    0,    0,   12,   12,   12,   12,   12,   12,   12,
       12,   12,    0,   34,   36,   24,    0,    0,    0,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   36,   23,
       20,    0,   12,   12,   12,   12,   12,   12,   12,   12,

       12,   28,   21,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,   12,   12,   12,
       12,   12,   12,   12,   12,   12,   12,    2,   12,   12,
       12,   12,   12,   12,   12,   12,   12,    3,    8,    4,
        6,   12,   12,    5,    1,    7,    9,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        2,    2,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    4,    1,    5,    6,    7,    8,    9,   10,
       11,   12,   13,   14,   15,   16,   17,   18,   19,   19,
       19,   19,   19,   19,   19,   19,   19,   20,   21,   22,
       23,   24,   25,    1,   26,   27,   28,   29,   30,   31,
       32,   33,   34,   33,   33,   35,   36,   37,   38,   39,
       33,   40,   41,   42,   43,   44,   33,   45,   46,   33,
       47,   48,   49,   50,   51,    1,   52,   52,   52,   53,

       54,   55,   33,   33,   56,   33,   33,   57,   33,   58,
       33,   33,   33,   33,   59,   33,   60,   33,   33,   61,
       33,   33,   62,   63,   64,   65,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[66] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    1,
        1,    1,    1,    1,    3,    3,    3,    4,    4,    1,
        1,    1,    1,    1,    1,    5,    5,    5,    5,    5,
        5,    6,    6,    6,    6,    6,    6,    6,    6,    6,
        6,    6,    6,    6,    6,    6,    1,    1,    1,    1,
        6,    5,    5,    5,    5,    6,    6,    6,    6,    6,
        6,    1,    1,    1,    1
    } ;

static yyconst flex_int16_t yy_base[256] =
    {   0,
        0,    0,   65,    0,  129,  133,  457,  458,  458,  458,
      458,    0,    0,    0,  458,  410,  425,   95,  420,  422,
      409,  411,  413,  458,  458,  426,  118,  125,  400,  458,
      458,  424,  124,  458,  128,  137,  137,  145,  154,  422,
      458,  159,  422,  121,  458,    0,  435,  458,  458,  420,
      388,  458,  135,  458,  458,  458,  165,  174,  103,  157,
        0,    0,  402,  409,  413,  410,  402,  407,  393,  406,
      404,  458,  458,  458,  458,  458,  153,  162,  458,  458,
      458,  458,  458,  458,  416,  185,  458,  458,  458,  191,
      214,  210,  212,    0,  235,  458,  458,  458,  408,  458,

      458,  458,  407,    0,  381,  458,  373,  458,  458,  183,
      134,  372,  458,  458,    0,  400,  391,  389,  388,  380,
      391,  381,  368,  388,  458,  458,  243,  458,  245,  264,
      458,  224,  175,  272,  251,  458,  458,  361,  357,  362,
      125,  373,  376,  369,  368,  380,  378,  365,  367,  375,
      177,  278,  297,  458,  194,  286,  458,  263,  347,  350,
      347,  348,  348,  359,  369,  372,  368,  356,  344,  352,
      355,  341,  199,  307,  337,  458,  323,  308,  307,  309,
      308,  316,  327,  305,  315,  303,  315,  321,  298,  458,
      458,  295,  318,  317,  317,  295,  304,  314,  311,  297,

      302,    0,  458,  302,  299,  281,  300,  287,  292,  290,
      271,  293,  283,  288,  264,  259,  257,  249,  245,  244,
      234,  230,  220,  207,  208,  210,  194,    0,  203,  195,
      194,  193,  195,  197,  190,  189,  167,    0,    0,    0,
        0,  159,  142,    0,    0,    0,    0,  458,  364,  366,
      370,  376,  379,  382,  385
    } ;

static yyconst flex_int16_t yy_def[256] =
    {   0,
      248,    1,  248,    3,  249,  249,  248,  248,  248,  248,
      248,  250,  251,  251,  248,  251,  251,  251,  251,  251,
      251,  251,  251,  248,  248,  248,  248,  248,  252,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  253,  253,  248,  248,  248,
      253,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      254,  251,  251,  251,  251,  251,  251,  251,  251,  251,
      251,  248,  248,  248,  248,  248,  252,  252,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  255,  248,  248,  248,  248,  248,  248,

      248,  248,  248,  253,  252,  248,  253,  248,  248,  248,
      248,  248,  248,  248,  254,  251,  251,  251,  251,  251,
      251,  251,  251,  251,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  255,  248,  248,  253,  248,  248,
      248,  251,  251,  251,  251,  251,  251,  251,  251,  251,
      248,  248,  248,  248,  248,  248,  248,  248,  253,  248,
      248,  248,  248,  251,  251,  251,  251,  251,  251,  251,
      251,  251,  248,  248,  253,  248,  248,  248,  248,  251,
      251,  251,  251,  251,  251,  251,  251,  251,  253,  248,
      248,  248,  251,  251,  251,  251,  251,  251,  251,  251,

      251,  253,  248,  251,  251,  251,  251,  251,  251,  251,
      251,  251,  251,  251,  251,  251,  251,  251,  251,  251,
      251,  251,  251,  251,  251,  251,  251,  251,  251,  251,
      251,  251,  251,  251,  251,  251,  251,  251,  251,  251,
      251,  251,  251,  251,  251,  251,  251,    0,  248,  248,
      248,  248,  248,  248,  248
    } ;

static yyconst flex_int16_t yy_nxt[524] =
    {   0,
        8,    9,   10,    8,   11,   12,    8,    8,    8,    8,
        8,    8,    8,    8,   13,   13,   14,   13,   13,    8,
        8,    8,   15,    8,    8,   13,   16,   13,   17,   18,
       19,   13,   13,   20,   13,   13,   13,   21,   13,   13,
       22,   13,   23,   13,   13,   13,    8,    8,    8,    8,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,    8,    8,    8,    8,   24,   24,   25,   26,   24,
       24,   27,   28,   29,   30,   31,   32,   33,   34,   35,
       36,   37,   38,   39,   40,   41,   42,   43,   44,   45,
       46,   46,   46,   46,   46,   46,   46,   46,   46,   47,

       46,   46,   46,   46,   46,   46,   46,   46,   46,   46,
       46,   48,   24,   49,   50,   46,   46,   51,   46,   46,
       46,   46,   46,   46,   46,   46,   52,   53,   54,   55,
       57,   58,   75,   59,   57,   58,   80,   59,   65,   66,
       73,   74,   82,  102,  103,   60,   81,   76,   87,   60,
       83,   84,   85,   88,   86,   86,  111,  108,  112,   89,
       90,  125,   91,   91,  248,   97,  110,  110,  113,   90,
       77,   95,   95,  114,   92,  110,  110,  162,   98,   93,
       99,  100,  163,   92,  110,  110,  247,   93,   93,   94,
      139,  140,  134,  134,  152,  152,   93,  109,   92,  246,

       78,   93,   86,   86,   93,   94,  245,   92,  129,  129,
       93,  156,  156,   93,  127,  128,  174,  174,  244,  128,
      130,  131,  133,  243,  133,  131,  242,  134,  134,   90,
      241,   91,   91,  240,  239,  238,  237,  236,  127,  128,
      235,  128,  234,   92,  130,  131,   93,  131,  132,  233,
       90,  232,   95,   95,   93,  151,  132,  151,  132,  231,
      152,  152,  129,  129,   92,  230,  132,   92,   93,   93,
      132,   93,  229,  132,  153,  154,  155,   93,  155,  154,
      132,  156,  156,  132,  228,  158,  227,  226,   92,  134,
      134,   93,  225,  158,   93,  152,  152,  158,  153,  154,

      224,  154,  157,  156,  156,  158,  157,  158,  128,  173,
      158,  173,  128,  223,  174,  174,  131,  222,  221,  158,
      131,  220,  158,  219,  174,  174,  157,  218,  157,  217,
      216,  215,  128,  214,  128,  213,  212,  154,  211,  210,
      131,  154,  131,  209,  208,  207,  206,  205,  204,  203,
      202,  201,  200,  199,  198,  197,  196,  195,  194,  193,
      192,  154,  191,  154,   56,   56,   56,   56,   56,   56,
       61,   61,   62,   62,   62,   62,   77,  190,   77,   77,
       77,   77,  104,  104,  104,  115,  115,  115,  135,  135,
      189,  188,  187,  186,  185,  184,  183,  182,  181,  180,

      179,  178,  177,  176,  175,  172,  171,  170,  169,  168,
      167,  166,  165,  164,  161,  160,  159,  150,  149,  148,
      147,  146,  145,  144,  143,  142,  141,  138,   78,  137,
      136,  126,  124,  123,  122,  121,  120,  119,  118,  117,
      116,  107,  106,  105,  101,   96,   79,   78,   72,   71,
       70,   69,   68,   67,   64,   63,  248,    7,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,

      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248
    } ;

static yyconst flex_int16_t yy_chk[524] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,

        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        5,    5,   28,    5,    6,    6,   33,    6,   18,   18,
       27,   27,   35,   44,   44,    5,   33,   28,   37,    6,
       35,   35,   36,   37,   36,   36,   59,   53,   59,   37,
       38,   77,   38,   38,   78,   42,   57,   57,   60,   39,
       78,   39,   39,   60,   38,   58,   58,  141,   42,   38,
       42,   42,  141,   39,  110,  110,  243,   38,   39,   38,
      111,  111,  133,  133,  151,  151,   39,   53,   38,  242,

       77,   38,   86,   86,   38,   38,  237,   39,   90,   90,
       39,  155,  155,   39,   86,   86,  173,  173,  236,   86,
       90,   90,   92,  235,   92,   90,  234,   92,   92,   91,
      233,   91,   91,  232,  231,  230,  229,  227,   86,   86,
      226,   86,  225,   91,   90,   90,   93,   90,   91,  224,
       95,  223,   95,   95,   93,  127,   91,  127,  132,  222,
      127,  127,  129,  129,   95,  221,  132,   91,   93,   95,
       91,   93,  220,   91,  129,  129,  130,   95,  130,  129,
      132,  130,  130,  132,  219,  135,  218,  217,   95,  134,
      134,   95,  216,  135,   95,  152,  152,  158,  129,  129,

      215,  129,  134,  156,  156,  158,  134,  135,  152,  153,
      135,  153,  152,  214,  153,  153,  156,  213,  212,  158,
      156,  211,  158,  210,  174,  174,  134,  209,  134,  208,
      207,  206,  152,  205,  152,  204,  201,  174,  200,  199,
      156,  174,  156,  198,  197,  196,  195,  194,  193,  192,
      189,  188,  187,  186,  185,  184,  183,  182,  181,  180,
      179,  174,  178,  174,  249,  249,  249,  249,  249,  249,
      250,  250,  251,  251,  251,  251,  252,  177,  252,  252,
      252,  252,  253,  253,  253,  254,  254,  254,  255,  255,
      175,  172,  171,  170,  169,  168,  167,  166,  165,  164,

      163,  162,  161,  160,  159,  150,  149,  148,  147,  146,
      145,  144,  143,  142,  140,  139,  138,  124,  123,  122,
      121,  120,  119,  118,  117,  116,  112,  107,  105,  103,
       99,   85,   71,   70,   69,   68,   67,   66,   65,   64,
       63,   51,   50,   47,   43,   40,   32,   29,   26,   23,
       22,   21,   20,   19,   17,   16,    7,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,

      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248,  248,  248,  248,  248,  248,  248,  248,
      248,  248,  248
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int yy_flex_debug;
int yy_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "parser.l"
#line 9 "parser.l"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h> 
#include <dirent.h>
#include <errno.h>
#include <assert.h>
#include "parser.h"
#include "y.tab.h"

#define MAX_BUF_SIZE 1 << 15 //32K

typedef enum {
    CONFIG_CMD_NONE,
    SRC_ROOT_DIR,
    OUTPUT_DIR,
    UNDEF_FLAGS,
    DEFINE_FLAGS,
    IFDEF_REMOVE,
    EVALUATE_NUM,
	ASSIGN_VALUE,
	EXCLUDE_FILES,
	BYPASS_FILES,
	FILTER_SUFFIX,
} CONFIG_CMD;

extern YYSTYPE yylval;
FILE_INFO *g_include_files = NULL;
void count();
int column = 0;
int lineno = 1;
int ifdef_remove=0;
int evaluate_num=0;
static int ignore(int);
static int comment(int);
static char *save_to_NL();
static IFDEF_STACK *ifdef_top = NULL;
typedef struct stringstack
{
    char *string;
    struct stringstack *next;
} StringStack;

static StringStack *ifdef_string = 0, *else_string = 0, *endif_string = 0;
static StringStack *exclude_files = 0;
static StringStack *bypass_files = 0;
static StringStack *filter_suffix = 0;
static void push_ifdef_string(char *s);
static void push_else_string(char *s);
static void push_endif_string(char *s);
static void add_exclude_files(char *fname);
static void add_bypass_files(char *fname);
static void add_filter_suffix(char *suffix);
static int in_exclude_files(char *fname);
static int in_bypass_files(char *fname);
static int in_filter_suffix(char *suffix);
static int create_dir_stack(char *src_dir_name, char *dst_dir_name, char bAll); 
static int config_stage = 1;
static int config_eq    = 0;
static CONFIG_CMD config_cmd = CONFIG_CMD_NONE;
static char *assign_variable = 0;
typedef struct symbol
{
    char *name;
    struct symbol *next;
}SYMBOL;

static SYMBOL *g_remove_flags = 0;
static char *g_src_root_dir = 0, *g_output_dir = 0;

typedef struct flist
{
    FILE *src;
    FILE *dst;
    char *src_name;
    char *dst_name;
	char isBypass;
    struct flist *next;
} FLIST;

static FLIST *g_file_list = 0;

#line 742 "lex.yy.c"

#define INITIAL 0
#define PARSER 1
#define PLAINTEXT 2

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy (void );

int yyget_debug (void );

void yyset_debug (int debug_flag  );

YY_EXTRA_TYPE yyget_extra (void );

void yyset_extra (YY_EXTRA_TYPE user_defined  );

FILE *yyget_in (void );

void yyset_in  (FILE * in_str  );

FILE *yyget_out (void );

void yyset_out  (FILE * out_str  );

int yyget_leng (void );

char *yyget_text (void );

int yyget_lineno (void );

void yyset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap (void );
#else
extern int yywrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, yyleng, 1, yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int yylex (void);

#define YY_DECL int yylex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
#line 94 "parser.l"

#line 933 "lex.yy.c"

	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer(yyin,YY_BUF_SIZE );
		}

		yy_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of yytext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 249 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 458 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 95 "parser.l"
{ count(); config_cmd = SRC_ROOT_DIR; }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 96 "parser.l"
{ count(); config_cmd = OUTPUT_DIR;   }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 97 "parser.l"
{ count(); config_cmd = UNDEF_FLAGS; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 98 "parser.l"
{ count(); config_cmd = DEFINE_FLAGS; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 99 "parser.l"
{ count(); config_cmd = IFDEF_REMOVE; }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 100 "parser.l"
{ count(); config_cmd = EVALUATE_NUM; }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 101 "parser.l"
{ count(); config_cmd = EXCLUDE_FILES; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 102 "parser.l"
{ count(); config_cmd = BYPASS_FILES; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 103 "parser.l"
{ count(); config_cmd = FILTER_SUFFIX; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 104 "parser.l"
{ count(); config_cmd = ASSIGN_VALUE; assign_variable=strdup(yytext+1); }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 105 "parser.l"
{ count(); config_eq = 1; }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 106 "parser.l"
{ 
                          count();  
                          if (!config_eq)
                          {
                              printf("Skip token: %s\n", yytext);
                              return;
                          }
                          switch(config_cmd)
                          {
                              case SRC_ROOT_DIR:
                                  g_src_root_dir = strdup(yytext);
                                  printf("SRC ROOT=%s\n", g_src_root_dir);
                                  break;
                              case OUTPUT_DIR:
                                  g_output_dir = strdup(yytext);
                                  printf("OUTPUT =%s\n", g_output_dir);
                                  break;
                              case UNDEF_FLAGS:
                              {
                                  create_symbol(strdup(yytext), 0);
                                  printf("UNDEF FLAGS=%s\n", yytext);
                                  break;
                              }
                              case DEFINE_FLAGS:
                              {
                                  create_symbol(strdup(yytext), 1);
                                  printf("DEFINE FLAGS=%s\n", yytext);
                                  break;
                              }
                              case IFDEF_REMOVE:
                              {
								  if (!strcmp(yytext, "y") || !strcmp(yytext, "Y"))
									  ifdef_remove = 1;

                                  printf("IFDEF_REMOVE=%s\n", yytext);
                                  break;
                              }
                              case EVALUATE_NUM:
                              {
								  if (!strcmp(yytext, "y") || !strcmp(yytext, "Y"))
									  evaluate_num = 1;

                                  printf("EVALUATE_NUM=%s\n", yytext);
                                  break;
                              }
                              case EXCLUDE_FILES:
                              {
								  add_exclude_files(yytext);
                                  printf("Add EXCLUDE_FILES=%s\n", yytext);
                                  break;
                              }
                              case BYPASS_FILES:
                              {
								  add_bypass_files(yytext);
                                  printf("Add BYPASS_FILES=%s\n", yytext);
                                  break;
                              }
                              case FILTER_SUFFIX:
                              {
								  add_filter_suffix(yytext);
                                  printf("Add FILTER_SUFFIX=%s\n", yytext);
                                  break;
                              }

                              case ASSIGN_VALUE:
                              {
							      int value;
								  if (!assign_variable)
								  {
									  printf("ERROR! no assign variable at L%d\n", lineno);
								  } 
								  else
								  {
								      value = strtol(yytext, NULL, 10); 
                                      assign_symbol(assign_variable, create_scalar_expr(value));
                                      printf("ASSIGN_VALUE:%s=%d\n", assign_variable, value);
									  assign_variable = 0;
								  }
                                  break;
                              }
                              default:
                                  printf("Unknown token: %s\n", yytext);
                                  break;
                          }
                        }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 191 "parser.l"
{ count(); ignore(0); }
	YY_BREAK
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
#line 192 "parser.l"
{ count(); }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 193 "parser.l"
{ count(); }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 194 "parser.l"
{ count(); }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 196 "parser.l"
{ return comment(1); }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 197 "parser.l"
{ return ignore(1); }
	YY_BREAK
case 19:
/* rule 19 can match eol */
YY_RULE_SETUP
#line 198 "parser.l"
{ count(); yylval.string = strdup(yytext); return(STRING); }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 199 "parser.l"
{ count(); BEGIN PARSER; push_ifdef_string(save_to_NL()); return(IFDEF); }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 200 "parser.l"
{ count(); BEGIN PARSER; push_ifdef_string(save_to_NL()); return(IFNDEF); }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 201 "parser.l"
{ count(); BEGIN PARSER; push_ifdef_string(save_to_NL()); return(IF2); }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 202 "parser.l"
{ count(); BEGIN PARSER; push_endif_string(save_to_NL()); return(ENDIF); }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 203 "parser.l"
{ count(); BEGIN PARSER; push_else_string(save_to_NL()); return(ELSE2); }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 204 "parser.l"
{ count(); return ignore(1); }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 205 "parser.l"
{ comment(0); }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 206 "parser.l"
{ ignore(0); }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 207 "parser.l"
{ count();  return(DEFINED); }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 208 "parser.l"
{ count(); yylval.num = strtol(yytext, NULL, 16); return(CONSTANT); }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 209 "parser.l"
{ count(); yylval.num = strtol(yytext, NULL, 8);  return(CONSTANT); }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 210 "parser.l"
{ count(); yylval.num = strtol(yytext, NULL, 10); return(CONSTANT); }
	YY_BREAK
case 32:
/* rule 32 can match eol */
YY_RULE_SETUP
#line 211 "parser.l"
{ count(); yylval.num = 0; return(CONSTANT); }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 212 "parser.l"
{ count(); yylval.num = strtol(yytext, NULL, 10);  return(CONSTANT); }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 213 "parser.l"
{ count(); yylval.real = strtod(yytext, NULL); return(CONSTANT); }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 214 "parser.l"
{ count(); yylval.real = strtod(yytext, NULL); return(CONSTANT); }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 215 "parser.l"
{ count();  yylval.string = strdup(yytext); return(IDENTIFIER);}
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 216 "parser.l"
{ count(); return('('); }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 217 "parser.l"
{ count(); return(')'); }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 218 "parser.l"
{ count(); return('<'); }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 219 "parser.l"
{ count(); return('>'); }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 220 "parser.l"
{ count(); return('!'); }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 221 "parser.l"
{ count(); return(AND_OP); }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 222 "parser.l"
{ count(); return(OR_OP); }
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 223 "parser.l"
{ count(); return(ELLIPSIS); }
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 224 "parser.l"
{ count(); return(RIGHT_ASSIGN); }
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 225 "parser.l"
{ count(); return(LEFT_ASSIGN); }
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 226 "parser.l"
{ count(); return(ADD_ASSIGN); }
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 227 "parser.l"
{ count(); return(SUB_ASSIGN); }
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 228 "parser.l"
{ count(); return(MUL_ASSIGN); }
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 229 "parser.l"
{ count(); return(DIV_ASSIGN); }
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 230 "parser.l"
{ count(); return(MOD_ASSIGN); }
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 231 "parser.l"
{ count(); return(AND_ASSIGN); }
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 232 "parser.l"
{ count(); return(XOR_ASSIGN); }
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 233 "parser.l"
{ count(); return(OR_ASSIGN); }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 234 "parser.l"
{ count(); return(RIGHT_OP); }
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 235 "parser.l"
{ count(); return(LEFT_OP); }
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 236 "parser.l"
{ count(); return(INC_OP); }
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 237 "parser.l"
{ count(); return(DEC_OP); }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 238 "parser.l"
{ count(); return(PTR_OP); }
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 239 "parser.l"
{ count(); return(LE_OP); }
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 240 "parser.l"
{ count(); return(GE_OP); }
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 241 "parser.l"
{ count(); return(EQ_OP); }
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 242 "parser.l"
{ count(); return(NE_OP); }
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 243 "parser.l"
{ count(); return(';'); }
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 244 "parser.l"
{ count(); return('{'); }
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 245 "parser.l"
{ count(); return('}'); }
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 246 "parser.l"
{ count(); return(','); }
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 247 "parser.l"
{ count(); return(':'); }
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 248 "parser.l"
{ count(); return('='); }
	YY_BREAK
case 70:
YY_RULE_SETUP
#line 249 "parser.l"
{ count(); return('['); }
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 250 "parser.l"
{ count(); return(']'); }
	YY_BREAK
case 72:
YY_RULE_SETUP
#line 251 "parser.l"
{ count(); return('.'); }
	YY_BREAK
case 73:
YY_RULE_SETUP
#line 252 "parser.l"
{ count(); return('&'); }
	YY_BREAK
case 74:
YY_RULE_SETUP
#line 253 "parser.l"
{ count(); return('~'); }
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 254 "parser.l"
{ count(); return('-'); }
	YY_BREAK
case 76:
YY_RULE_SETUP
#line 255 "parser.l"
{ count(); return('+'); }
	YY_BREAK
case 77:
YY_RULE_SETUP
#line 256 "parser.l"
{ count(); return('*'); }
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 257 "parser.l"
{ count(); return('/'); }
	YY_BREAK
case 79:
YY_RULE_SETUP
#line 258 "parser.l"
{ count(); return('%'); }
	YY_BREAK
case 80:
YY_RULE_SETUP
#line 259 "parser.l"
{ count(); return('^'); }
	YY_BREAK
case 81:
YY_RULE_SETUP
#line 260 "parser.l"
{ count(); return('|'); }
	YY_BREAK
case 82:
YY_RULE_SETUP
#line 261 "parser.l"
{ count(); return('?'); }
	YY_BREAK
case 83:
/* rule 83 can match eol */
YY_RULE_SETUP
#line 262 "parser.l"
{ count(); BEGIN PLAINTEXT; }
	YY_BREAK
case 84:
YY_RULE_SETUP
#line 263 "parser.l"
{ count(); }
	YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(PARSER):
case YY_STATE_EOF(PLAINTEXT):
#line 264 "parser.l"
{ return EOF; }
	YY_BREAK
case 85:
YY_RULE_SETUP
#line 266 "parser.l"
ECHO;
	YY_BREAK
#line 1535 "lex.yy.c"

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (yytext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (yytext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (yytext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( yywrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (yytext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(yytext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (yytext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (yytext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (yytext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (yytext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart(yyin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) ((yy_n_chars) + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = (yy_n_chars) + number_to_move + ((yy_n_chars) >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) yyrealloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size  );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(yytext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);

	for ( yy_cp = (yytext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 249 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 249 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 248);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up yytext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(yytext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (yytext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart(yyin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (yytext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve yytext */
	(yy_hold_char) = *++(yy_c_buf_p);

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void yyrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer(yyin,YY_BUF_SIZE );
	}

	yy_init_buffer(YY_CURRENT_BUFFER,input_file );
	yy_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void yy_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(yytext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * 
 */
    void yy_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree((void *) b->yy_ch_buf  );

	yyfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	yy_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void yy_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack();

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void yypop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void yyensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );
								  
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)yyrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		if ( ! (yy_buffer_stack) )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yyalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (yyconst char * yystr )
{
    
	return yy_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) yyalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = (yy_hold_char); \
		(yy_c_buf_p) = yytext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int yyget_lineno  (void)
{
        
    return yylineno;
}

/** Get the input stream.
 * 
 */
FILE *yyget_in  (void)
{
        return yyin;
}

/** Get the output stream.
 * 
 */
FILE *yyget_out  (void)
{
        return yyout;
}

/** Get the length of the current token.
 * 
 */
int yyget_leng  (void)
{
        return yyleng;
}

/** Get the current token.
 * 
 */

char *yyget_text  (void)
{
        return yytext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void yyset_lineno (int  line_number )
{
    
    yylineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  in_str )
{
        yyin = in_str ;
}

void yyset_out (FILE *  out_str )
{
        yyout = out_str ;
}

int yyget_debug  (void)
{
        return yy_flex_debug;
}

void yyset_debug (int  bdebug )
{
        yy_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}

/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state();
	}

	/* Destroy the stack itself. */
	yyfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *yyrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void yyfree (void * ptr )
{
	free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 266 "parser.l"


yywrap()
{
    return(1);
}

void change_file()
{
	//printf("change file: g_file_list=%p\n", g_file_list);
    // output the parsed file
    if (g_file_list)
    {
        output_file(g_file_list->dst);
        //printf("src fp=%p, dst fp=%p\n", g_file_list->src, g_file_list->dst);
        fclose(g_file_list->src);
        fclose(g_file_list->dst);
        g_file_list = g_file_list->next;
    }

    if (config_stage)
    {
        config_stage = 0;
        BEGIN PLAINTEXT;
        create_dir_stack(g_src_root_dir, g_output_dir, 0);
    }

    // switch to the new file
    yy_delete_buffer(YY_CURRENT_BUFFER);

	while(g_file_list && g_file_list->isBypass)
	{
		char buf[520];
		sprintf(buf, "\\cp -f %s %s", 
				g_file_list->src_name, 
				g_file_list->dst_name);
		system(buf);
		g_file_list = g_file_list->next;
        printf("\nWARN: bypass %sn", g_file_list->src_name);
	}

    if (g_file_list)
    {

        FILE *fsrc;
        FILE *fdst;
		fsrc = fopen(g_file_list->src_name, "r");
		fdst = fopen(g_file_list->dst_name, "w");
		g_file_list->src = fsrc;
		g_file_list->dst = fdst;
        yyin = g_file_list->src;
        printf("\n====BEGIN parsing %s =====\n", g_file_list->src_name);
        lineno = 0;
        yy_switch_to_buffer(yy_create_buffer(yyin,YY_BUF_SIZE));
		BEGIN PLAINTEXT;
        return;
    }
	yyin = 0;
}

char *current_file_name()
{
    if (!g_file_list) 
        return 0;
    return g_file_list->src_name;
}
//#define DBG_PUSH_POP
char *pop_ifdef_string()
{
    char *s;
    if (!ifdef_string)
    {
        printf("ERROR! no ifdef string to pop, at L(%d)\n", lineno);
        return 0;
    }
    s = ifdef_string->string;
    ifdef_string = ifdef_string->next;
#ifdef DBG_PUSH_POP
    printf("pop ifdef string:%s\n", s);
#endif
    return s;
}
char *pop_else_string()
{
    char *s;
    if (!else_string)
    {
        printf("ERROR! no else string to pop, at L(%d)\n", lineno);
        return 0;
    }
    s = else_string->string;

    else_string = else_string->next;
#ifdef DBG_PUSH_POP
    printf("pop else string:%s\n", s);
#endif
    return s;
}
char *pop_endif_string()
{
    char *s;
    if (!endif_string)
    {
        printf("ERROR! no endif string to pop, at L(%d)\n", lineno);
        return 0;
    }
    s = endif_string->string;
    endif_string = endif_string->next;
#ifdef DBG_PUSH_POP
    printf("pop endif string:%s\n", s);
#endif
    return s;
}
void push_ifdef_string(char *s)
{
    StringStack *ss = (StringStack *)malloc(sizeof(StringStack));
    ss->next = ifdef_string;
    ss->string = s;
#ifdef DBG_PUSH_POP
    printf("push ifdef string %s\n", s);
#endif
    ifdef_string = ss;
}
void push_else_string(char *s)
{
    StringStack *ss = (StringStack *)malloc(sizeof(StringStack));
    ss->next = else_string;
    ss->string = s;
#ifdef DBG_PUSH_POP
    printf("push else string %s\n", s);
#endif
    else_string = ss;
}
void push_endif_string(char *s)
{
    StringStack *ss = (StringStack *)malloc(sizeof(StringStack));
    ss->next = endif_string;
    ss->string = s;
#ifdef DBG_PUSH_POP
    printf("push endif string:%s\n", s);
#endif
    endif_string = ss;
}

static void add_exclude_files(char *fname)
{
	StringStack *ss = (StringStack *)malloc(sizeof(StringStack));
	ss->string = strdup(fname);
	ss->next = exclude_files;
	exclude_files = ss;
}

static void add_bypass_files(char *fname)
{
	StringStack *ss = (StringStack *)malloc(sizeof(StringStack));
	ss->string = strdup(fname);
	ss->next = bypass_files;
	bypass_files = ss;
}

static void add_filter_suffix(char *suffix)
{
	StringStack *ss = (StringStack *)malloc(sizeof(StringStack));
	ss->string = strdup(suffix);
	ss->next = filter_suffix;
	filter_suffix = ss;
}

static int in_exclude_files(char *fname)
{
	StringStack *ss = exclude_files;
	while(ss)
	{
		if (!strcmp(ss->string, fname))
			return 1;
				
		ss = ss->next;
	}
	return 0;
}

static int in_bypass_files(char *fname)
{
	StringStack *ss = bypass_files;
	while(ss)
	{
		if (!strcmp(ss->string, fname))
			return 1;
				
		ss = ss->next;
	}
	return 0;
}

static int in_filter_suffix(char *fname)
{
	StringStack *ss = filter_suffix;
	int flen = strlen(fname);
	char *suffix = (char *)(fname+flen);

	while(suffix > fname)
	{
		if (*(--suffix) == '.')
			break;
	}

	if (suffix == fname) 
		return 0; 

	while(ss)
	{
	    
		if (!strcmp(ss->string, suffix))
		{
	        //printf("suffix===>%s\n", suffix);
			return 1;
		}
				
		ss = ss->next;
	}
	return 0;
}

#if 0
int process_file(char *fname)
{
    FILE_INFO *node = malloc(sizeof(FILE_INFO));
    char out_name[128];
    FILE *fp = NULL, *fout = NULL;
    if (!fname)
    {
        printf("\nERROR! L%d: include NULL file name.\n", lineno);
        return 0;
    }
    if (!strcmp(fname, "STDIN"))
    {
        fp = stdin;
    } else {
        fp = fopen(fname, "r"); 
        if (!fp)
        {
            printf("ERROR, can't open %s for read\n", fname);
            return 0;
        }
    }
    sprintf(out_name, "%s.out", fname);
    fout = fopen(out_name, "w"); 
    if (!fout)
    {
        printf("ERROR, can't open %s for write\n", out_name);
        return 0;
    }

    yyin = fp;
    node->fp = fp;
    node->fout = fout;
    node->fname = strdup(fname);
    //node->saved_state = YY_CURRENT_BUFFER;

    g_include_files = node;
    lineno=1;
    
    yy_switch_to_buffer(yy_create_buffer(yyin,YY_BUF_SIZE));
    return 1;
}
#endif
static int
comment(int retString)
{
    char c, c1;
    char *buf, *p;

	buf = malloc(MAX_BUF_SIZE);
	memset(buf, 0, MAX_BUF_SIZE);
	p = buf;

    strcpy(p, yytext);
    p += yyleng;
loop:

    while ((c = input()) != '*' && c != 0)
    {
        if (c == '\n') lineno++;

        *(p++) = c;
        //putchar(c);
    }
    if (c == '*')
        *(p++) = c;

    if ((c1 = input()) != '/' && c != 0)
    {
        unput(c1);
        goto loop;
    }

    if (c1 == '/')
        *(p++) = c1;
        //putchar(c1);

    *p = 0x00;

	if (p-buf > MAX_BUF_SIZE)
	{
		printf("ERROR, internal buffer overflow:"
				"%d > %d, at %s,L(%d)\n", (p-buf), MAX_BUF_SIZE,
				current_file_name(), lineno);
		assert(p-buf <= MAX_BUF_SIZE);
	}

    if (retString)
    {
        yylval.string = strdup(buf);
		free(buf);
        return STRING;
    }
	free(buf);
    return 0;
}

static int ignore(int retString)
{
    char c;
    char *buf, *p;

	buf = malloc(MAX_BUF_SIZE);
    memset(buf, 0, MAX_BUF_SIZE);
	p = buf;
    strcpy(p, yytext);
    p += yyleng;

loop:
    while ((c = input()) != '\n' && c != 0 && c != EOF)
    {
        *(p++) = c;
        //putchar(c);
    }


	// don't unput EOF, which is identified as any char 
	// "." => ignore(1) => infinite loop
	if (c != EOF) 
		unput(c);

    *p = 0x00;

	if (p-buf > MAX_BUF_SIZE)
	{
		printf("ERROR, internal buffer overflow:"
				"%d > %d, at %s,L(%d)\n", (p-buf), MAX_BUF_SIZE,
				current_file_name(), lineno);
		assert(p-buf <= MAX_BUF_SIZE);
	}

    if (retString)
    {
        yylval.string = strdup(buf);
        //printf("\nignore: [%s]\n", yylval.string);
		free(buf);
        return (STRING);
    }
	free(buf);
    return 0;

}

static char *save_to_NL()
{
    char c, prev;
    char *buf, *p, *start;
	int unput_NL = 0;

	buf = malloc(MAX_BUF_SIZE);
    memset(buf, 0, MAX_BUF_SIZE);
	p = buf;
    strcpy(p, yytext);
    p += yyleng;
    start = p;

loop:
    while ((c = input()) != '\n' && c != 0 && c != EOF )
    {
        *(p++) = c;
        //putchar(c);
		prev = c;
    }
	if (c != EOF)
		*(p++) = c;
	if (c == '\n' && prev == '\\')
		goto loop;

    *p = 0x00;

	if (p-buf > MAX_BUF_SIZE)
	{
		printf("ERROR, internal buffer overflow:"
				"%d > %d, at %s,L(%d)\n", (p-buf), MAX_BUF_SIZE,
				current_file_name(), lineno);
		assert(p-buf <= MAX_BUF_SIZE);
	}


	unput_NL=0;
    while(p > start)
	{
		char uc = *--p;

		// only unput 1st NL
		if (uc == '\n' && !unput_NL)
		{
			unput(uc);
			//printf("unput [%c:%x]\n", uc, uc);
			unput_NL = 1;
		}
		else if (uc != '\n' && uc != '\\')
		{
			unput(uc);
			//printf("unput [%c:%x]\n", uc, uc);
		}

	}

	//printf("===========\n");
	p = strdup(buf);
	free(buf);

    return p;
}



void count()
{
    int i;

    for (i = 0; yytext[i] != '\0'; i++)
        if (yytext[i] == '\n')
        {
            //printf("\nline:%d\n", lineno);
            lineno++;
            column = 0;
        }
        else if (yytext[i] == '\t')
            column += 8 - (column % 8);
        else
            column++;

    //ECHO;
}

static 
int create_dir_stack(char *src_dir_name, char *dst_dir_name, char bypassAll) {
    DIR *pDIR;
    struct dirent *pDirEnt;

    /* Open the current directory */
    pDIR = opendir(src_dir_name);

    if ( pDIR == NULL ) {
        fprintf( stderr, "%s %d: opendir(%s) failed (%s)\n",
                 __FILE__, __LINE__, src_dir_name, strerror( errno ));
        exit( -1 );
    }

    /* Get each directory entry from pDIR and print its name */

    pDirEnt = readdir( pDIR );
    //printf("====== SRC src ====\n");
    while ( pDirEnt != NULL ) {
		char isBypass = bypassAll;
		if (in_exclude_files(pDirEnt->d_name))
		{
			printf("WARN: exclude file/dir=%s\n", pDirEnt->d_name);
			pDirEnt = readdir( pDIR );
			continue;
		}
		if (in_bypass_files(pDirEnt->d_name))
		{
			printf("WARN: bypass file/dir=%s\n", pDirEnt->d_name);
			isBypass = 1;
		}
        if (strcmp(pDirEnt->d_name, ".") && strcmp(pDirEnt->d_name, ".."))
        {
            char src_path[256], dst_path[256];
            int  slen = strlen(src_dir_name);
            int  dlen = strlen(dst_dir_name);
            if (src_dir_name[slen-1] == '/')
                sprintf(src_path, "%s%s", src_dir_name, pDirEnt->d_name);
            else
                sprintf(src_path, "%s/%s", src_dir_name, pDirEnt->d_name);

            if (dst_dir_name[dlen-1] == '/')
                sprintf(dst_path, "%s%s", dst_dir_name, pDirEnt->d_name);
            else
                sprintf(dst_path, "%s/%s", dst_dir_name, pDirEnt->d_name);
            //printf("src_path=%s\n", src_path);
            //if (flen > 2)
            //    printf("suffix=%s\n", &pDirEnt->d_name[flen-2]);
            if(is_dir(src_path, dst_path))
            {
                char buf[275]; 
                sprintf(buf, "mkdir -p %s", dst_path);
                system(buf);

                create_dir_stack(src_path, dst_path, isBypass);
                printf("src dir=> %s\n", src_path);
                printf("dst dir=> %s\n", dst_path);
            }
            else if (in_filter_suffix(pDirEnt->d_name))
				/*
                (!strcmp(&pDirEnt->d_name[flen-2], ".c") ||
                 !strcmp(&pDirEnt->d_name[flen-2], ".h")))
				 */
            {
                //printf("file=>%s\n", pDirEnt->d_name);
                FILE *fsrc = fopen(src_path, "r");
                FILE *fdst = fopen(dst_path, "w");
                if (!fsrc)
                    printf("ERROR! can't open %s for read\n", src_path);
                if (!fdst)
                    printf("ERROR! can't open %s for write\n", dst_path);
                if (fsrc && fdst)
                {
                    FLIST *flist = malloc(sizeof(FLIST));
					fclose(fsrc);
					fclose(fdst);
                    flist->src   = 0; //fsrc;
                    flist->dst   = 0; //fdst;
                    flist->src_name = strdup(src_path);
                    flist->dst_name = strdup(dst_path);
                    flist->isBypass = isBypass;
                    flist->next  = g_file_list;
                    g_file_list  = flist;
                }
            }
			else
			{
                char buf[520]; 

				// copy file from src to dst directly
                sprintf(buf, "\\cp -f %s %s", src_path, dst_path);
				system(buf);
                //printf("%s\n", buf);
			}
        }
        pDirEnt = readdir( pDIR );
    }

    /* Release the open directory */

    closedir( pDIR );

    return 0;
}

int is_dir(const char * dirname)    
{    
    struct stat sDir; 
    if (stat(dirname ,&sDir) < 0 )    
        return 0; 
    if (S_IFDIR == (sDir.st_mode & S_IFMT))    
        return 1; 
    return 0; 
} 


